/*
 * The SmartWeb Framework
 * Copyright (C) 2004-2006
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 *
 * For further informations on the SmartWeb Framework please visit
 *
 *                        http://smartweb.sourceforge.net
 */
package net.smartlab.web.test;

import java.io.File;
import java.io.FileInputStream;
import java.net.URL;
import java.sql.DriverManager;

import net.smartlab.web.BusinessObjectFactory;
import net.smartlab.web.config.ConfigurationStrategy;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.dbunit.DatabaseTestCase;
import org.dbunit.database.DatabaseConfig;
import org.dbunit.database.DatabaseConnection;
import org.dbunit.database.IDatabaseConnection;
import org.dbunit.dataset.DefaultDataSet;
import org.dbunit.dataset.IDataSet;
import org.dbunit.dataset.xml.FlatXmlDataSet;
import org.dbunit.operation.DatabaseOperation;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.dialect.Dialect;
import org.hibernate.dialect.DialectFactory;

/**
 * Base class for <code>Model</code> testing.
 * 
 * @author rlogiacco
 * @deprecated model tests should not involve domain or factory testing
 */
public abstract class ModelTestCase extends DatabaseTestCase {

	/**
	 * Provides logging for the test cases.
	 */
	protected Log logger = LogFactory.getLog(this.getClass());

	private DatabaseConnection connection;


	/**
	 * Default constructor.
	 */
	public ModelTestCase() {
		super();
	}

	/**
	 * Named constructor.
	 * 
	 * @param name test name.
	 */
	public ModelTestCase(String name) {
		super(name);
	}

	/**
	 * @see junit.framework.TestCase#setUp()
	 */
	protected void setUp() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("setUp() - start");
		}
		BusinessObjectFactory.setConfigurationStrategy(new TestConfiguration());
		super.setUp();
	}

	/**
	 * @see junit.framework.TestCase#tearDown()
	 */
	protected void tearDown() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("tearDown() - start");
		}
		BusinessObjectFactory.close();
		super.tearDown();
	}

	/**
	 * Returns the configuration file to be used for test purposes. The default
	 * implementation uses <code>res/smartweb-test.hcf</code> but the method
	 * could be overriden to support different configuration files for different
	 * tests.
	 * 
	 * @return the configuration file to be used for test purposes.
	 * @throws Exception if the file open fails.
	 */
	protected URL getConfigurationFile() throws Exception {
		return new File("res/smartweb.jar.hcf").toURL();
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getConnection()
	 */
	protected IDatabaseConnection getConnection() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("getConnection() - start");
		}
		if (connection == null || connection.getConnection().isClosed()) {
			Configuration configuration = new Configuration().configure(getConfigurationFile());
			String driver = configuration.getProperty("hibernate.connection.driver_class");
			String url = configuration.getProperty("hibernate.connection.url");
			String username = configuration.getProperty("hibernate.connection.username");
			String password = configuration.getProperty("hibernate.connection.password");
			String dialectName = configuration.getProperty("dialect");
			Dialect dialect = DialectFactory.buildDialect(dialectName);
			Class.forName(driver);
			connection = new DatabaseConnection(DriverManager.getConnection(url, username, password));
			connection.getConfig().setFeature(DatabaseConfig.FEATURE_QUALIFIED_TABLE_NAMES, dialect.qualifyIndexName());
			connection.getConfig().setProperty(DatabaseConfig.PROPERTY_ESCAPE_PATTERN,
					dialect.openQuote() + "?" + dialect.closeQuote());
		}
		return connection;
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getDataSet()
	 */
	protected IDataSet getDataSet() throws Exception {
		if (logger.isDebugEnabled()) {
			logger.debug("getDataSet() - start");
			logger.trace("  res = " + this.getDataSetFile());
		}
		try {
			return new FlatXmlDataSet(new FileInputStream(this.getDataSetFile()));
		} catch (Exception e) {
			logger.warn("getDataSet() - not found");
			return new DefaultDataSet();
		}
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getSetUpOperation()
	 */
	protected DatabaseOperation getSetUpOperation() throws Exception {
		return DatabaseOperation.REFRESH;
	}

	/**
	 * @see org.dbunit.DatabaseTestCase#getTearDownOperation()
	 */
	protected DatabaseOperation getTearDownOperation() throws Exception {
		return DatabaseOperation.NONE;
	}

	/**
	 * TODO documentation
	 * 
	 * @return
	 */
	protected abstract String getDataSetFile();


	/**
	 * TODO documentation
	 * 
	 * @author rlogiacco
	 */
	protected class TestConfiguration implements ConfigurationStrategy {

		private SessionFactory factory = null;


		/**
		 * @see net.smartlab.web.config.ConfigurationStrategy#getSessionFactory(net.smartlab.web.BusinessObjectFactory)
		 */
		public SessionFactory getSessionFactory(BusinessObjectFactory bof) {
			if (factory == null) {
				try {
					Configuration configuration = new Configuration().configure(getConfigurationFile());
					String driver = configuration.getProperty("hibernate.connection.driver_class");
					String url = configuration.getProperty("hibernate.connection.url");
					String username = configuration.getProperty("hibernate.connection.username");
					String password = configuration.getProperty("hibernate.connection.password");
					Class.forName(driver);
					connection = new DatabaseConnection(DriverManager.getConnection(url, username, password));
					factory = configuration.buildSessionFactory();
				} catch (Exception e) {
					logger.error(e);
				}
			}
			return factory;
		}
	}
}
